/*
 * n_rhole_c_suppression.h
 * Define class N_rhole_C_suppression, class for potentials
 * Nuclear potential is Rectangular hole (finite) plus
 * emulation of Coulomb (see below),
 * Coulomb potential is point-Coulomb interaction with exponential suppression
 * (de-facto, Yukawa potential).
 *
 * Such double-Coulomb made in aim to compute various cases
 * using the same binary files.
 *
 * Copyright (c) 2010, 2011 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_N_RHOLE_C_SUPPRESSION_H
#define SCHQ_N_RHOLE_C_SUPPRESSION_H

// **********************************************
# include <string>
# include <stdexcept>
# include <sstream>
# include <cmath>

# include <libconfig.h++>

# include "schq_potentials_base.h"
# include "schq_physical_system_base.h"

// **********************************************
namespace schq
{

  // **********************************************
  class N_rhole_C_suppression : public Potentials_base
  {
  private:
    // Nuclear
    std::string nuclear_type;
    // Rectanguar hole potential parameters
    double depth;
    double hole_radius;
    int Z1; // Coulomb-like part
    int Z2;
    double cutoff_radius;

    // Coulomb
    std::string Coulomb_type;
    // Coulomb cut off radius
    double C_left_cutoff_radius;
    double C_right_cutoff_radius;

    // **********************************************
    /*
     * void initialise_nuclear(const libconfig::Setting & nuclear)
     * Initialise rectangular hole nuclear interaction potential.
     */
    void initialise_nuclear(const libconfig::Setting & nuclear)
    {
      get_log().debug("initialise_nuclear(): start.");

      std::string tmp = nuclear["type"];
      nuclear_type = tmp;
      depth = nuclear["depth"];
      hole_radius = nuclear["hole_radius"];
      Z1 = nuclear["Z1"];
      Z2 = nuclear["Z2"];
      cutoff_radius = nuclear["cutoff_radius"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "Nuclear interaction potential: " << nuclear_type;
      log4cpp::eol(ost);
      ost << "V_0: " << depth;
      log4cpp::eol(ost);
      ost << "R: " << hole_radius;
      log4cpp::eol(ost);
      ost << "Z1: " << Z1;
      log4cpp::eol(ost);
      ost << "Z2: " << Z2;
      log4cpp::eol(ost);
      ost << "cut off radius: " << cutoff_radius;
      log4cpp::eol(ost);

      get_log().debug("initialise_nuclear(): finish.");
    } // initialise_nuclear()

    // **********************************************
    /*
     * void initialise_Coulomb(const libconfig::Setting & Coulomb)
     * Initialise Coulomb interaction potential.
     */
    void initialise_Coulomb(const libconfig::Setting & Coulomb)
    {
      get_log().debug("initialise_Coulomb(): start.");

      std::string tmp = Coulomb["type"];
      Coulomb_type = tmp;
      C_left_cutoff_radius = Coulomb["left_cutoff_radius"];
      C_right_cutoff_radius = Coulomb["right_cutoff_radius"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "Coulomb interaction potential: " << Coulomb_type;
      log4cpp::eol(ost);
      ost << "Left cut off radius: " << C_left_cutoff_radius;
      log4cpp::eol(ost);
      ost << "Right cut off radius: " << C_right_cutoff_radius;
      log4cpp::eol(ost);

      get_log().debug("initialise_Coulomb(): finish.");
    } // initialise_Coulomb()

    // **********************************************
    /*
     * void make_potentials(const libconfig::Setting & potentials)
     * Initialise potentials.
     */
    void make_potentials(const libconfig::Setting & potentials)
    {
      get_log().debug("make_potentials(): start.");
      initialise_nuclear(potentials["nuclear"]);
      initialise_Coulomb(potentials["Coulomb"]);
      get_log().debug("make_potentials(): finish.");
    } // make_potentials()

    // **********************************************
  public:
    /*
     * Constructor.
     */
  N_rhole_C_suppression(const libconfig::Setting & potentials,
			const log4cpp::Category & uplevel_log) :
    Potentials_base(potentials, uplevel_log)
      {
	get_log().debug("N_rhole_C_cut(): start.");
	make_potentials(potentials);
	get_log().debug("N_rhole_C_cut(): finish.");
      } // N_rhole_C_suppression()

    // **********************************************
    /*
     * Destructor.
     */
    virtual ~N_rhole_C_suppression()
      {
	// Nothing to do.
      }

    // **********************************************
    /*
     * double dVN(const double r, const Physical_system_base & sys) const
     * Returns value of derivative of nuclear interaction
     * potential at point r.
     */
    double dVN(const double r, const Physical_system_base & sys) const
    {
      double retval = 0;
      const double RN = hole_radius;
      const double RC = cutoff_radius;
      if( (r > RN) && (r < RC) )
	{
	  const double alpha = sys.alpha();
	  const double hc = sys.hc();
	  retval = (-1);
	  retval *= Z1 * Z2;
	  retval *= alpha;
	  retval *= hc;
	  retval /= std::pow(r, 2);
	}
      return retval;
    } // dVN()

    // **********************************************
    /*
     * double dVC(const double r, const Physical_system_base & sys) const
     * Returns value of derivative of Coulomb interaction
     * potential at point r.
     */
    double dVC(const double r, const Physical_system_base & sys) const
    {
      double retval = 0;
      const double left = C_left_cutoff_radius;
      const double right = C_right_cutoff_radius;
      if( (r > left) && (r < right) )
	{
	  const Physical_system_base::Particles_pair & particles =
	    sys.particles();
	  const double alpha = sys.alpha();
	  const double hc = sys.hc();
	  retval = (-1);
	  retval *= particles.first.Z();
	  retval *= particles.second.Z();
	  retval *= alpha;
	  retval *= hc;
	  retval /= std::pow(r, 2);
	}
      return retval;
    } // dVC()

    // **********************************************
    /*
     * double k0_momentum(const Physical_system_base & sys) const
     * Returns value of k0.
     */
    double k0_momentum(const Physical_system_base & sys) const
    {
      double retval;
      const double mu = sys.mu();
      const double hc = sys.hc();
      retval = std::sqrt(2 * mu * depth) / hc;
      return retval;
    } // k0_momentum()

    // **********************************************
    /*
     * double VN(const double r, const Physical_system_base & sys) const
     * Returns value of nuclear interaction potential at point r.
     */
    double VN(const double r, const Physical_system_base & sys) const
    {
      if( r < hole_radius ) return (-depth);

      double retval = 0;
      const double RC = cutoff_radius;
      const double RN = hole_radius;
      if( (r > RN) && (r < RC) )
	{
	  const double alpha = sys.alpha();
	  const double hc = sys.hc();
	  retval = Z1 * Z2;
	  retval *= alpha;
	  retval *= hc;
	  retval /= r;
	}
      return retval;
    } // VN()

    // **********************************************
    /*
     * double VC(const double r, const Physical_system_base & sys) const
     * Returns value of Coulomb interaction potential at point r.
     */
    double VC(const double r, const Physical_system_base & sys) const
    {
      double retval = 0;
      const double left = C_left_cutoff_radius;
      const double right = C_right_cutoff_radius;
      if( (r > left) && (r < right) )
	{
	  const Physical_system_base::Particles_pair & particles =
	    sys.particles();
	  const double alpha = sys.alpha();
	  const double hc = sys.hc();
	  retval = particles.first.Z();
	  retval *= particles.second.Z();
	  retval *= alpha;
	  retval *= hc;
	  retval /= r;
	}
      return retval;
    } // VC()
  }; // class N_rhole_C_suppression

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_N_RHOLE_C_SUPPRESSION_H

// **********************************************
// End of file.
// **********************************************
