/*
 * schq_bound_bloodhound.h
 * Define class Bound_bloodhound
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_BOUND_BLOODHOUND_H
#define SCHQ_BOUND_BLOODHOUND_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solver_base.h"
# include "schq_solver_rkf45.h"

// **********************************************
namespace schq
{
  class Bound_bloodhound : public Bookworm
  {
  private:
    bool output_wf_to_file;
    std::string wave_function_filename;
    Physical_system_base * sys_val;
    Solver_base * solver_val;
    double energy;
    double step;
    int max_accuracy_factor;
    int step_limit;
    double sew_point;

    // ****************************************************
    /*
     * const Solution_tabulated & energy_rolldown()
     */
    const Solution_tabulated & energy_rolldown()
      {
	get_log().debug("energy_rolldown(): start.");
	int cnt; // Step counter
	double d = 0.0; // Wronskians
	double d_prev = 0.0;
	bool fall = false;

	for(cnt = 0; cnt < step_limit; cnt++)
	  {
	    solver().clear();
	    system().set_energy(energy);
	    {
	      std::ostringstream ost;
	      ost << "energy_rolldown(): E = " << energy;
	      std::string msg = ost.str();
	      get_log().debug(msg);
	    }
	    Solution_tabulated & solution = solver().solve();
	    d_prev = d;
	    d = rolldown_stop_criterion(solution);
	    if( (cnt > 0) && (sgn(d) != sgn(d_prev)) )
	      {
		fall = true;
		get_log().debug("energy_rolldown(): falled into pit!");
		break;
	      }
	    energy += step;
	  }
	energy = system().E();

	if( ! fall)
	  {
	    std::ostringstream ost;
	    ost << __FILE__ << "(" << __LINE__
		<< "): energy_rolldown(): Can not find bound state. "
		<< "Stopped at E = " << energy;
	    std::string msg = ost.str();
	    get_log().fatal(msg);
	    throw new std::runtime_error(msg);
	  }

	get_log().debug("energy_rolldown(): finish.");
	return solver().solution();
      } // energy_rolldown()

    // ****************************************************
    /*
     * const Solution_tabulated & find_bound_state()
     */
    const Solution_tabulated & find_bound_state()
      {
	get_log().debug("find_bound_state(): start.");
	int acc_factor;
	for(acc_factor = 0; acc_factor < max_accuracy_factor; acc_factor++)
	  {
	    energy_rolldown();
	    // Exception throwed if bound state was not found
	    step /= 2; // Increasing accuracy
	    energy -= 3 * step; // Rollback
	    {
	      std::ostringstream ost;
	      ost << "find_bound_state(): rollback: E = " << energy
		  << " MeV; step = " << step << " MeV.";
	      std::string msg = ost.str();
	      get_log().debug(msg);
	    }
	  }

	energy = system().E();
	{
	  std::ostringstream ost;
	  ost << "find_bound_state(): bound state hunt success at E = "
	      << energy << " MeV; system energy is set to E_binding.";
	  std::string msg = ost.str();
	  get_log().notice(msg);
	}

	get_log().debug("find_bound_state(): finish.");
	return solver().solution();
      } // find_bound_state()

    // ****************************************************
    /*
     * double rolldown_stop_criterion(Solution_tabulated & sol)
     */
    double rolldown_stop_criterion(Solution_tabulated & sol)
    {
      const Physical_system_base & sys = system();
      double u1 = sol.interpolate(sew_point);
      double u2 = sol.interpolate_deriv(sew_point);
      double y1 = sys.WC(sew_point);
      double y2 = sys.dWC(sew_point);
      double z1 = sys.WCN(sew_point);
      double z2 = sys.dWC(sew_point);
      double retval = u1 * y2 - u2 * y1;
      retval /= z1 * y2 - z2 * y1;
      return retval;
    } // rolldown_stop_criterion()

    // ****************************************************
    /*
     * bool sgn(double x) const
     * Returns true if x >= 0 and false otherwise.
     */
    bool sgn(double x) const
      {
	return (x >= 0);
      }

    // ****************************************************
    /*
     * Solver_base & solver()
     */
    Solver_base & solver()
      {
	return *solver_val;
      }

    // ****************************************************
    /*
     * Physical_system_base & system()
     */
    Physical_system_base & system()
      {
	return *sys_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Bound_bloodhound(const libconfig::Setting & root,
		   const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runbloodhound")
      {
	get_log().debug("Bound_bloodhound(): start.");
	try
	  {
	    libconfig::Setting & sys = root["system"];
	    // typedef Physical_system MUST be defined at upper level
	    sys_val = new Physical_system(sys, get_log());

	    libconfig::Setting & bh = root["bloodhound"];
	    libconfig::Setting & num = bh["solver"];
	    solver_val = new Solver_rkf45(sys_val, num, get_log());

	    double sp = bh["sew_point"];
	    sew_point = sp;

	    libconfig::Setting & area = bh["energy_area"];
	    double st = area["start"];
	    energy = st;
	    double init_stp = area["initial_step"];
	    step = init_stp;
	    int acc_f = area["max_accuracy_factor"];
	    max_accuracy_factor = acc_f;
	    int lim = area["step_limit"];
	    step_limit = lim;

	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["wave_function"];
		wave_function_filename = tmp;
		output_wf_to_file = true;

		// Final print
		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output wave-function: "
		    << wave_function_filename;
		log4cpp::eol(ost);
	      }
	    else
	      {
		output_wf_to_file = false;
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Bound_bloodhound(): finish.");
      } // Bound_bloodhound()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Bound_bloodhound()
    {
      delete solver_val;
      delete sys_val;
    } // ~Bound_bloodhound()

    // ****************************************************
    /*
     * void run()
     * Solve the Schrodinger equation and make other things.
     */
    void run()
    {
      get_log().debug("run(): start.");

      const Solution_tabulated & solution = find_bound_state();
      get_log().notice("Bound state found.");

      if(output_wf_to_file)
	{
	  get_log().debug("Write obtained solution to file.");
	  std::ofstream wf_out(wave_function_filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << solution;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & wf =
	root.add("wave_function", libconfig::Setting::TypeGroup);
      solver().solution().add_to_setting(wf);
      libconfig::Setting & be =
	root.add("binding_energy", libconfig::Setting::TypeFloat);
      be = (-1) * system().E();
    } // to_setting_as_children()
  }; // class Bound_bloodhound
} // namespace schq

// **********************************************
#endif // SCHQ_BOUND_BLOODHOUND_H

// **********************************************
// End of file.
// **********************************************
