/*
 * schq_bound_comparator.h
 * Define class Bound_comparator
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_BOUND_COMPARATOR_H
#define SCHQ_BOUND_COMPARATOR_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solution_tabulated.h"

// **********************************************
namespace schq
{
  class Bound_comparator : public Bookworm
  {
  private:
    bool output_to_file;
    std::string filename;
    Physical_system_base * sys_val;
    Solution_tabulated wave_function;
    Solution_tabulated comparision;

    // ****************************************************
    /*
     * Physical_system_base & system()
     */
    Physical_system_base & system()
      {
	return *sys_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Bound_comparator(const libconfig::Setting & root,
		   const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runcompare")
      {
	get_log().debug("Bound_comparator(): start.");
	try
	  {
	    libconfig::Setting & wf = root["normalised_wave_function"];
	    wave_function.read_from_setting(wf);

	    libconfig::Setting & s = root["system"];
	    // typedef Physical_system MUST be defined above
	    sys_val = new Physical_system(s, get_log());

	    libconfig::Setting & be = root["binding_energy"];
	    double ben = be;
	    system().set_energy( (-1) * ben);

	    {
	      log4cpp::CategoryStream ost(get_log().debugStream());
	      ost << "Energy set to " << system().E();
	      log4cpp::eol(ost);
	    }
	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["comparision_results"];
		filename = tmp;
		output_to_file = true;

		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output comparision results: "
		    << filename;
		log4cpp::eol(ost);
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Bound_comparator(): finish.");
      } // Bound_comparator()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Bound_comparator()
    {
      delete sys_val;
    } // ~Bound_comparator()

    // ****************************************************
    /*
     * void run()
     * Compare wave-function with asymptotic expansion.
     */
    void run()
    {
      get_log().debug("run(): start.");

      const Solution_tabulated::Solution_table & wfdata = wave_function.data();
      Solution_tabulated::Solution_table::const_iterator i;

      // Computations
      for(i = wfdata.begin(); i != wfdata.end(); i++)
	{
	  double r = (*i).first;
	  double u = (*i).second;
	  double a = system().WC(r);
	  double c = u / a;
	  comparision.push_back(Solution_tabulated::Solution_point(r, c));
	}

      if(output_to_file)
	{
	  get_log().debug("Write obtained results to file.");
	  std::ofstream wf_out(filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << comparision;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & wf =
	root.add("comparision_results", libconfig::Setting::TypeGroup);
      comparision.add_to_setting(wf);
      libconfig::Setting & be =
	root.add("binding_energy", libconfig::Setting::TypeFloat);
      be = (-1) * system().E();
    } // to_setting_as_children()
  }; // Bound_comparator
} // namespace schq

// **********************************************
#endif // SCHQ_BOUND_COMPARATOR_H

// **********************************************
// End of file.
// **********************************************
