/*
 * schq_bound_g2_manager.h
 * Define class Bound_G2_manager
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_BOUND_G2_MANAGER_H
#define SCHQ_BOUND_G2_MANAGER_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"

// **********************************************
namespace schq
{
  class Bound_G2_manager : public Bookworm
  {
  private:
    bool output_to_file;
    std::string filename;
    Physical_system_base * sys_val;
    double anc;
    double g2;

    // ****************************************************
    /*
     * Physical_system_base & system()
     */
    Physical_system_base & system()
      {
	return *sys_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Bound_G2_manager(const libconfig::Setting & root,
		   const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "rung2")
      {
	get_log().debug("Bound_G2_manager(): start.");
	try
	  {
	    libconfig::Setting & s = root["system"];
	    // typedef Physical_system MUST be defined above
	    sys_val = new Physical_system(s, get_log());

	    libconfig::Setting & be = root["binding_energy"];
	    double ben = be;
	    system().set_energy( (-1) * ben);

	    libconfig::Setting & anc_sett = root["anc"];
	    anc = anc_sett;

	    {
	      log4cpp::CategoryStream ost(get_log().debugStream());
	      ost << "Energy set to " << system().E();
	      log4cpp::eol(ost);
	      ost << "ANC: " << anc;
	      log4cpp::eol(ost);
	    }

	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["g2_value"];
		filename = tmp;
		output_to_file = true;

		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output G^2: "
		    << filename;
		log4cpp::eol(ost);
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Bound_G2_manager(): finish.");
      } // Bound_G2_manager()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Bound_G2_manager()
    {
      delete sys_val;
    } // ~Bound_G2_manager()

    // ****************************************************
    /*
     * void run()
     */
    void run()
    {
      get_log().debug("run(): start.");

      double den = system().mu() / system().hc();
      den *= den;
      g2 = M_PI * anc*anc / den;
      {
	log4cpp::CategoryStream ost(get_log().noticeStream());
	ost << "G^2: " << g2;
	log4cpp::eol(ost);
      }

      if(output_to_file)
	{
	  get_log().debug("Write obtained results to file.");
	  std::ofstream wf_out(filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << g2;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & g2_sett =
	root.add("G2", libconfig::Setting::TypeFloat);
      g2_sett = g2;
    } // to_setting_as_children()
  }; // Bound_G2_manager
} // namespace schq

// **********************************************
#endif // SCHQ_BOUND_G2_MANAGER_H

// **********************************************
// End of file.
// **********************************************
