/*
 * schq_bound_normaliser_manager.h
 * Define class Bound_normaliser_manager
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_BOUND_NORMALISER_MANAGER_H
#define SCHQ_BOUND_NORMALISER_MANAGER_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_normaliser_base.h"
# include "schq_normaliser_bound.h"

// **********************************************
namespace schq
{
  class Bound_normaliser_manager : public Bookworm
  {
  private:
    bool output_wf_to_file;
    std::string wave_function_filename;
    Normaliser_base * normaliser_val;

    // ****************************************************
    /*
     * Normaliser_base & normaliser()
     */
    Normaliser_base & normaliser()
      {
	return *normaliser_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Bound_normaliser_manager(const libconfig::Setting & root,
			   const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runnormalise")
      {
	get_log().debug("Bound_normaliser_manager(): start.");
	try
	  {
	    libconfig::Setting & n = root["normaliser"];
	    libconfig::Setting & wf = root["wave_function"];
	    libconfig::Setting & s = root["system"];
	    libconfig::Setting & be = root["binding_energy"];
	    normaliser_val = new Normaliser_bound(n, s, wf, get_log());
	    double ben = be;
	    normaliser().system().set_energy( (-1) * ben);
	    {
	      log4cpp::CategoryStream ost(get_log().debugStream());
	      ost << "Energy set to " << normaliser().system().E();
	      log4cpp::eol(ost);
	    }
	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["normalised_wave_function"];
		wave_function_filename = tmp;
		output_wf_to_file = true;

		// Final print
		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output normalised wave-function: "
		    << wave_function_filename;
		log4cpp::eol(ost);
	      }
	    else
	      {
		output_wf_to_file = false;
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Bound_normaliser_manager(): finish.");
      } // Bound_normaliser_manager()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Bound_normaliser_manager()
    {
      delete normaliser_val;
    } // ~Bound_normaliser_manager()

    // ****************************************************
    /*
     * void run()
     * Normalise wave-function.
     */
    void run()
    {
      get_log().debug("run(): start.");

      const Solution_tabulated & solution = normaliser().normalise();
      get_log().notice("Wave-function normalised.");

      if(output_wf_to_file)
	{
	  get_log().debug("Write obtained wave-function to file.");
	  std::ofstream wf_out(wave_function_filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << solution;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & wf =
	root.add("normalised_wave_function", libconfig::Setting::TypeGroup);
      normaliser().wave_function().add_to_setting(wf);
      libconfig::Setting & be =
	root.add("binding_energy", libconfig::Setting::TypeFloat);
      be = (-1) * normaliser().system().E();
    } // to_setting_as_children()
  }; // Bound_normaliser_manager
} // namespace schq

// **********************************************
#endif // SCHQ_BOUND_NORMALISER_MANAGER_H

// **********************************************
// End of file.
// **********************************************
