/*
 * schq_bound_rms_manager.h
 * Define class Bound_RMS_manager
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_BOUND_RMS_MANAGER_H
#define SCHQ_BOUND_RMS_MANAGER_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>
# include <cmath>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include <gsl/gsl_interp.h>
# include <gsl/gsl_spline.h>
# include <gsl/gsl_errno.h>

# include "schq_bookworm.h"
# include "schq_solution_tabulated.h"

// **********************************************
namespace schq
{
  class Bound_RMS_manager : public Bookworm
  {
  private:
    bool output_to_file;
    std::string filename;
    Solution_tabulated wave_function;
    double sew_point;
    double rms;

    // ****************************************************
    /*
     * double partial_MS_numerical() const
     * Evaluate numerical part of mean square radius (numerical integral)
     * *You need to get sqrt(retval) to get RMS!*
     */
    double partial_MS_numerical() const
    {
      get_log().debug("partial_MS_numerical(): start.");

      double retval;

      // Fill arrays
      const Solution_tabulated::Solution_table & wfdata = wave_function.data();
      Solution_tabulated::Solution_table::const_iterator i;
      const Solution_tabulated::Solution_table::size_type sz =
	wfdata.size() + 1;
      double xa[sz];
      double ya[sz];
      int j = 0;
      // This point is needed to make successful interpolation near origin
      xa[0] = 0.0;
      ya[0] = 0.0;
      j++;
      for(i = wfdata.begin(); i != wfdata.end(); i++)
	{
	  double x = (*i).first;
	  xa[j] = x;
	  double y = (*i).second;
	  ya[j] = (y*y) * (x*x);
	  j++;
	}

      // Preparations
      const gsl_interp_type * ityp = gsl_interp_cspline;
      double left = (*(wfdata.begin())).first;
      double right = (*(wfdata.rbegin())).first;
      right = sew_point; // *WARNING* At Irgaziev's Algorythm R_max is used!
      gsl_spline * squared = gsl_spline_alloc(ityp, sz);
      gsl_spline_init(squared, xa, ya, sz);
      gsl_interp_accel * acc = gsl_interp_accel_alloc();

      // Evaluate integral
      int status = gsl_spline_eval_integ_e(squared, left, right,
					   acc, &retval);

      // Cleanup
      gsl_interp_accel_free(acc);
      gsl_spline_free(squared);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): partial_MS_numerical(): "
	      << "gsl_spline_eval_integ_e() returned error! "
	      << "Status: " << gsl_strerror(status);
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      get_log().debug("partial_MS_numerical(): finish.");
      return retval;
    } // partial_MS_numerical()

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Bound_RMS_manager(const libconfig::Setting & root,
		    const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runrms")
      {
	get_log().debug("Bound_RMS_manager(): start.");
	try
	  {
	    libconfig::Setting & wf = root["normalised_wave_function"];
	    wave_function.read_from_setting(wf);

	    libconfig::Setting & rms = root["rms"];
	    sew_point = rms["sew_point"];

	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["rms_radius"];
		filename = tmp;
		output_to_file = true;

		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output RMS radius: "
		    << filename;
		log4cpp::eol(ost);
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Bound_RMS_manager(): finish.");
      } // Bound_RMS_manager()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Bound_RMS_manager()
    {
      // Nothing to do
    } // ~Bound_RMS_manager()

    // ****************************************************
    /*
     * void run()
     * Compare wave-function with asymptotic expansion.
     */
    void run()
    {
      get_log().debug("run(): start.");

      double ms; // Mean square radius <r^2>
      ms = partial_MS_numerical();
      rms = std::sqrt(ms);
      {
	log4cpp::CategoryStream ost(get_log().noticeStream());
	ost << "RMS radius: " << rms;
	log4cpp::eol(ost);
      }

      if(output_to_file)
	{
	  get_log().debug("Write obtained RMS radius to file.");
	  std::ofstream wf_out(filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << rms;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & rms_sett =
	root.add("rms_radius", libconfig::Setting::TypeFloat);
      rms_sett = rms;
    } // to_setting_as_children()
  }; // Bound_RMS_manager
} // namespace schq

// **********************************************
#endif // SCHQ_BOUND_RMS_MANAGER_H

// **********************************************
// End of file.
// **********************************************
