/*
 * schq_continuum_phaseshift.h
 * Define class Continuum_phaseshift
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_CONTINUUM_PHASESHIFT_H
#define SCHQ_CONTINUUM_PHASESHIFT_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solution_tabulated.h"

// **********************************************
namespace schq
{
  class Continuum_phaseshift : public Bookworm
  {
  private:
    bool output_to_file;
    std::string output_filename;
    Physical_system_base * sys_val;
    Solution_tabulated wf;
    double r1;
    double r2;

    // ****************************************************
    /*
     * double calculate_phase_shift()
     * Calculates phase shift in radians.
     */
    double calculate_phase_shift()
    {
      const Physical_system_base & sys = system();
      double F1 = sys.Coulomb_wave_F(sys.rho(r1));
      double G1 = sys.Coulomb_wave_G(sys.rho(r1));
      double F2 = sys.Coulomb_wave_F(sys.rho(r2));
      double G2 = sys.Coulomb_wave_G(sys.rho(r2));
      double u1 = wf.interpolate(r1);
      double u2 = wf.interpolate(r2);
      double x = u1/u2;
      double tg = (x * F2 - F1) / (G1 - x * G2);
      double retval = std::atan(tg);
      return retval;
    } // calculate_phase_shift()

    // ****************************************************
    /*
     * const Physical_system_base & system() const
     */
    const Physical_system_base & system() const
      {
	return * sys_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Continuum_phaseshift(const libconfig::Setting & root,
		       const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runphaseshift")
      {
	get_log().debug("Continuum_phaseshift(): start.");
	try
	  {
	    libconfig::Setting & ps = root["phase_shift_calculator"];
	    libconfig::Setting & points = ps["points"];
	    libconfig::Setting & wfs = root["normalised_wave_function"];
	    libconfig::Setting & s = root["system"];
	    // Typedef Physical_system MUST be defined earlier
	    sys_val = new Physical_system(s, get_log());
	    wf.read_from_setting(wfs);
	    double x1 = points["r1"];
	    double x2 = points["r2"];
	    r1 = x1;
	    r2 = x2;
	    log4cpp::CategoryStream ost(get_log().infoStream());
	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["phase_shift"];
		output_filename = tmp;
		output_to_file = true;

		ost << "File to output phase-shift: "
		    << output_filename;
		log4cpp::eol(ost);
	      }
	    else
	      {
		output_to_file = false;
	      }

	    ost << "r1 = " << r1;
	    log4cpp::eol(ost);
	    ost << "r2 = " << r2;
	    log4cpp::eol(ost);
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Continuum_phaseshift(): finish.");
      } // Continuum_phaseshift()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Continuum_phaseshift()
    {
      delete sys_val;
    } // ~Continuum_phaseshift()

    // ****************************************************
    /*
     * void run()
     * Normalise wave-function.
     */
    void run()
    {
      get_log().debug("run(): start.");

      log4cpp::CategoryStream ost(get_log().noticeStream());
      ost << "Phase-shift = " << calculate_phase_shift();
      log4cpp::eol(ost);

      if(output_to_file)
	{
	  get_log().debug("Write obtained wave-function to file.");
	  std::ofstream wf_out(output_filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << calculate_phase_shift() << std::endl;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & delta =
	root.add("phase_shift", libconfig::Setting::TypeFloat);
      delta = calculate_phase_shift();
    } // to_setting_as_children()
  }; // Continuum_phaseshift
} // namespace schq

// **********************************************
#endif // SCHQ_CONTINUUM_PHASESHIFT_H

// **********************************************
// End of file.
// **********************************************
