/*
 * schq_continuum_solver_manager.h
 * Define class Continuum_solver_manager
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_CONTINUUM_SOLVER_MANAGER_H
#define SCHQ_CONTINUUM_SOLVER_MANAGER_H

// **********************************************
# include <string>
# include <fstream>
# include <iostream>
# include <sstream>
# include <cstdio>
# include <stdexcept>
# include <typeinfo>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solver_base.h"
# include "schq_solver_rkf45.h"

// **********************************************
namespace schq
{
  class Continuum_solver_manager : public Bookworm
  {
  private:
    bool output_wf_to_file;
    std::string wave_function_filename;
    Physical_system_base * sys_val;
    Solver_base * solver_val;

    // ****************************************************
    /*
     * Solver_base & solver()
     */
    Solver_base & solver()
      {
	return *solver_val;
      }

    // ****************************************************
    /*
     * Physical_system_base & system()
     */
    Physical_system_base & system()
      {
	return *sys_val;
      }

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
  Continuum_solver_manager(const libconfig::Setting & root,
			   const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "runsolve")
      {
	get_log().debug("Continuum_solver_manager(): start.");
	try
	  {
	    libconfig::Setting & sys = root["system"];
	    // typedef Physical_system MUST be defined at upper level
	    sys_val = new Physical_system(sys, get_log());
	    libconfig::Setting & num = root["solver"];
	    solver_val = new Solver_rkf45(sys_val, num, get_log());
	    if(root.exists("fileoutput"))
	      {
		libconfig::Setting & output = root["fileoutput"];
		std::string tmp = output["wave_function"];
		wave_function_filename = tmp;
		output_wf_to_file = true;

		// Final print
		log4cpp::CategoryStream ost(get_log().infoStream());
		ost << "File to output wave-function: "
		    << wave_function_filename;
		log4cpp::eol(ost);
	      }
	    else
	      {
		output_wf_to_file = false;
	      }
	  }
	catch(libconfig::SettingException & e)
	  {
	    std::string msg = "SettingException catched! ";
	    msg += e.getPath();
	    msg += ": ";
	    msg += e.what();
	    get_log().error(msg);
	    throw e;
	  }
	get_log().debug("Continuum_solver_manager(): finish.");
      } // Continuum_solver_manager()

    // ****************************************************
    /*
     * Destructor.
     */
    ~Continuum_solver_manager()
    {
      delete solver_val;
      delete sys_val;
    } // ~Continuum_solver_manager()

    // ****************************************************
    /*
     * void run()
     * Solve the Schrodinger equation and make other things.
     */
    void run()
    {
      get_log().debug("run(): start.");

      const Solution_tabulated & solution = solver().solve();
      get_log().notice("Schroedinger equation solved.");

      if(output_wf_to_file)
	{
	  get_log().debug("Write obtained solution to file.");
	  std::ofstream wf_out(wave_function_filename.c_str());
	  wf_out.precision(10);
	  wf_out.setf(std::ios_base::scientific, std::ios_base::floatfield);
	  wf_out << solution;
	}

      get_log().debug("run(): finish.");
    } // run()

    // ****************************************************
    /*
     * void to_setting_as_children(libconfig::Setting & root)
     * Adds object settings as children of given root
     */
    void to_setting_as_children(libconfig::Setting & root)
    {
      libconfig::Setting & wf =
	root.add("wave_function", libconfig::Setting::TypeGroup);
      solver().solution().add_to_setting(wf);
    } // to_setting_as_children()
  }; // Continuum_solver_manager
} // namespace schq

// **********************************************
#endif // SCHQ_CONTINUUM_SOLVER_MANAGER_H

// **********************************************
// End of file.
// **********************************************
