/*
 * n_yukawa_c_sphere.h
 * Define class N_Yukawa_C_sphere, real class for physical system
 * with Yukawa nuclear interaction and Coulomb interaction like
 * charged sphere.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_N_YUKAWA_C_SPHERE_H
#define SCHQ_N_YUKAWA_C_SPHERE_H

// **********************************************
# include <string>
# include <stdexcept>
# include <cmath>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_potentials_base.h"
# include "schq_physical_system_base.h"

// **********************************************
namespace schq
{

  // **********************************************
  class N_Yukawa_C_sphere : public Potentials_base
  {
  private:
    // Nuclear
    std::string nuclear_type;
    // Yukawa potential parameters
    double depth;
    double inverse_radius;

    // Coulomb
    std::string Coulomb_type;
    // charged sphere radius
    double sphere_radius;

    // **********************************************
    /*
     * void initialise_nuclear(const libconfig::Setting & nuclear)
     * Initialise Yukawa nuclear interaction potential.
     */
    void initialise_nuclear(const libconfig::Setting & nuclear)
    {
      get_log().debug("initialise_nuclear(): start.");

      std::string tmp = nuclear["type"];
      nuclear_type = tmp;
      depth = nuclear["depth"];
      inverse_radius = nuclear["inverse_radius"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "Nuclear interaction potential: " << nuclear_type;
      log4cpp::eol(ost);
      ost << "V_0: " << depth;
      log4cpp::eol(ost);
      ost << "gamma: " << inverse_radius;
      log4cpp::eol(ost);

      get_log().debug("initialise_nuclear(): finish.");
    } // initialise_nuclear()

    // **********************************************
    /*
     * void initialise_Coulomb(const libconfig::Setting & Coulomb)
     * Initialise Coulomb charged sphere interaction potential.
     */
    void initialise_Coulomb(const libconfig::Setting & Coulomb)
    {
      get_log().debug("initialise_Coulomb(): start.");

      std::string tmp = Coulomb["type"];
      Coulomb_type = tmp;
      sphere_radius = Coulomb["radius"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "Coulomb interaction potential: " << Coulomb_type;
      log4cpp::eol(ost);
      ost << "sphere radius: " << sphere_radius;
      log4cpp::eol(ost);

      get_log().debug("initialise_Coulomb(): finish.");
    } // initialise_Coulomb()

    // **********************************************
    /*
     * void make_potentials(const libconfig::Setting & potentials)
     * Initialise potentials.
     */
    void make_potentials(const libconfig::Setting & potentials)
    {
      get_log().debug("make_potentials(): start.");
      initialise_nuclear(potentials["nuclear"]);
      initialise_Coulomb(potentials["Coulomb"]);
      get_log().debug("make_potentials(): finish.");
    } // make_potentials()

    // **********************************************
  public:
    /*
     * Constructor.
     */
  N_Yukawa_C_sphere(const libconfig::Setting & potentials,
		    const log4cpp::Category & uplevel_log) :
    Potentials_base(potentials, uplevel_log)
      {
	get_log().debug("N_Yukawa_C_sphere(): start.");
	make_potentials(potentials);
	get_log().debug("N_Yukawa_C_sphere(): finish.");
      } // N_Yukawa_C_sphere()

    // **********************************************
    /*
     * Destructor.
     */
    ~N_Yukawa_C_sphere()
      {
	// Nothing to do.
      }

    // **********************************************
    /*
     * double dVN(const double r, const Physical_system_base & sys) const
     * Returns value of derivative of nuclear interaction
     * potential at point r.
     */
    double dVN(const double r, const Physical_system_base & sys) const
    {
      double retval;
      const double gamma = inverse_radius;
      retval = depth * (1 + gamma * r);
      retval /= gamma * std::pow(r, 2) * std::exp(gamma * r);
      return retval;
    } // dVN()

    // **********************************************
    /*
     * double dVC(const double r, const Physical_system_base & sys) const
     * Returns value ofderivative of Coulomb interaction
     * potential at point r.
     */
    double dVC(const double r, const Physical_system_base & sys) const
    {
      double retval;
      const double RC = sphere_radius;
      const Physical_system_base::Particles_pair & particles =
	sys.particles();
      const double alpha = sys.alpha();
      const double hc = sys.hc();
      retval = (-1);
      retval *= particles.first.Z();
      retval *= particles.second.Z();
      retval *= alpha;
      retval *= hc;
      if(r < RC)
	{
	  retval *= r / std::pow(RC, 3);
	}
      else
	{
	  retval /= std::pow(r, 2);
	}
      return retval;
    } // dVC()

    // **********************************************
    /*
     * double k0_momentum(const Physical_system_base & sys) const
     * Returns value of k0.
     */
    double k0_momentum(const Physical_system_base & sys) const
    {
      double retval;
      const double mu = sys.mu();
      const double hc = sys.hc();
      retval = std::sqrt(2 * mu * depth) / hc;
      return retval;
    } // k0_momentum()

    // **********************************************
    /*
     * double VN(const double r, const Physical_system_base & sys) const
     * Returns value of nuclear interaction potential at point r.
     */
    double VN(const double r, const Physical_system_base & sys) const
    {
      double retval;
      const double rho = inverse_radius * r;
      retval = (-depth) / (rho * std::exp(rho));
      return retval;
    } // VN()

    // **********************************************
    /*
     * double VC(const double r, const Physical_system_base & sys) const
     * Returns value of Coulomb interaction potential at point r.
     */
    double VC(const double r, const Physical_system_base & sys) const
    {
      double retval;
      const double RC = sphere_radius;
      const Physical_system_base::Particles_pair & particles =
	sys.particles();
      const double alpha = sys.alpha();
      const double hc = sys.hc();
      retval = particles.first.Z();
      retval *= particles.second.Z();
      retval *= alpha;
      retval *= hc;
      if(r < RC)
	{
	  retval *= (3 - std::pow(r/RC, 2)) / (2 * RC);
	}
      else
	{
	  retval /= r;
	}
      return retval;
    } // VC()
  }; // class N_Yukawa_C_sphere

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_N_YUKAWA_C_SPHERE_H

// **********************************************
// End of file.
// **********************************************
