/*
 * normaliser_base.h
 * Define class Normaliser_base.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_NORMALISER_BASE_H
#define SCHQ_NORMALISER_BASE_H

// **********************************************
# include <gsl/gsl_interp.h>
# include <gsl/gsl_spline.h>

# include <log4cpp/Category.hh>

# include <libconfig.h++>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solution_tabulated.h"

// **********************************************
namespace schq
{
  class Normaliser_base : public Bookworm
  {
  protected:
    Physical_system_base * sys_val;
    bool normalised;
    Solution_tabulated primordial_wf;
    Solution_tabulated normalised_wf;

    // ****************************************************
  public:
    /*
     * Constructor.
     */
  Normaliser_base(libconfig::Setting & normaliser,
		  libconfig::Setting & system,
		  libconfig::Setting & wave_f,
		  log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "normaliser")
    {
      get_log().debug("Normaliser_base(): start.");
      normalised = false;
      primordial_wf.read_from_setting(wave_f);
      // Typedef Physical_system MUST be defined earlier.
      sys_val = new Physical_system(system, get_log());
      get_log().debug("Normaliser_base(): finish.");
    } // Normaliser_base()

    // ****************************************************
    /*
     * Destructor.
     */
    virtual ~Normaliser_base()
    {
      delete sys_val;
    } // ~Normaliser_base()

    // ****************************************************
    /*
     * Solution_tabulated & wave_function()
     * Returns the normalised wave-function.
     */
    Solution_tabulated & wave_function()
    {
      return normalised_wf;
    }

    // ****************************************************
    /*
     * const Solution_tabulated & wave_function() const
     * Returns the normalised wave-function.
     */
    const Solution_tabulated & wave_function() const
    {
      return normalised_wf;
    }

    // ****************************************************
    /*
     * bool is_normalised() const
     */
    bool is_normalised() const
    {
      return normalised;
    }

    // ****************************************************
    /*
     * const Solution_tabulated & normalise()
     * Returns the normalised wave-function.
     * Here it is pure virtual method.
     * Derivative classes MUST redefine it!
     */
    virtual const Solution_tabulated & normalise() = 0;

    // ****************************************************
    /*
     * Physical_system_base & system()
     */
    Physical_system_base & system()
    {
      return * sys_val;
    }

    // ****************************************************
    /*
     * const Physical_system_base & system() const
     */
    const Physical_system_base & system() const
    {
      return * sys_val;
    }
  }; // class Normaliser_base
} // namespace schq

// **********************************************
#endif // SCHQ_NORMALISER_BASE_H

// **********************************************
// End of file.
// **********************************************
