/*
 * normaliser_bound.h
 * Define class Normaliser_bound.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_NORMALISER_BOUND_H
#define SCHQ_NORMALISER_BOUND_H

// **********************************************
# include <cmath>

# include <libconfig.h++>

# include <gsl/gsl_interp.h>
# include <gsl/gsl_spline.h>
# include <gsl/gsl_errno.h>

# include "schq_solver_base.h"
# include "schq_normaliser_base.h"
# include "schq_physical_system_base.h"

// **********************************************
namespace schq
{
  class Normaliser_bound : public Normaliser_base
  {
  private:
    double sew_point;

    /*
     * double partial_N2_numerical() const
     * Evaluate numerical part of square norm (numerical integral)
     */
    double partial_N2_numerical() const
    {
      get_log().debug("partial_N2_numerical(): start.");

      double retval;

      // Fill arrays
      const Solution_tabulated::Solution_table prim_table =
	primordial_wf.data();
      Solution_tabulated::Solution_table::const_iterator i =
	prim_table.begin();
      const Solution_tabulated::Solution_table::size_type sz =
	prim_table.size() + 1;
      double xa[sz];
      double ya[sz];
      int j = 0;
      // This point is needed to make successful interpolation near origin
      xa[0] = 0.0;
      ya[0] = 0.0;
      j++;
      while(i != prim_table.end())
	{
	  xa[j] = (*i).first;
	  double y = (*i).second;
	  ya[j] = std::abs(y*y);
	  j++;
	  i++;
	}

      // Preparations
      const gsl_interp_type * ityp = gsl_interp_cspline;
      double left = (*(prim_table.begin())).first;
      double right = (*(prim_table.rbegin())).first;
      right = sew_point; // *WARNING* At Irgaziev's Algorythm R_max is used!
      gsl_spline * squared = gsl_spline_alloc(ityp, sz);
      gsl_spline_init(squared, xa, ya, sz);
      gsl_interp_accel * acc = gsl_interp_accel_alloc();

      // Evaluate integral
      int status = gsl_spline_eval_integ_e(squared, left, right,
					   acc, &retval);

      // Cleanup
      gsl_interp_accel_free(acc);
      gsl_spline_free(squared);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): partial_N2_numerical(): "
	      << "gsl_spline_eval_integ_e() returned error! "
	      << "Status: " << gsl_strerror(status);
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      get_log().debug("partial_N2_numerical(): finish.");
      return retval;
    } // partial_N2_numerical()

    // ****************************************************
    /*
     * double partial_N2_tail() const
     * Evaluate "tail" part of square norm (analythical
     * integral)
     */
    double partial_N2_tail()
    {
      double retval;
      const Physical_system_base & sys = system();
      const double ANC = primordial_wf.interpolate(sew_point) /
	sys.WC(sew_point);

      const double rho_sew = sys.rho(sew_point);
      gsl_sf_result gamma_inc;
      const int status = gsl_sf_gamma_inc_e((1.0 - 2.0 * sys.eta()),
					    2.0 * rho_sew, &gamma_inc);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): partial_N2_tail(): "
	      << "gsl_sf_gamma_inc_e() returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      retval = gamma_inc.val;
      retval *= ANC * ANC;
      retval /= 2.0 * sys.momentum();

      return retval;
    } // partial_N2_tail()
    
    // ****************************************************
  public:
    /*
     * Constructor.
     */
  Normaliser_bound(libconfig::Setting & normaliser,
		   libconfig::Setting & system,
		   libconfig::Setting & wave_f,
		   log4cpp::Category & uplevel_log) :
    Normaliser_base(normaliser, system, wave_f, uplevel_log)
    {
      get_log().debug("Normaliser_bound(): start.");

      sew_point = normaliser["sew_point"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "sew point: " << sew_point;
      log4cpp::eol(ost);

      get_log().debug("Normaliser_base(): finish.");
    } // Normaliser_bound()

    // ****************************************************
    /*
     * Destructor.
     */
    virtual ~Normaliser_bound()
    {
      // Nothing to do!
    } // ~Normaliser_bound()

    // ****************************************************
    /*
     * Solution_tabulated & normalise()
     * Returns the normalised wave-function.
     */
    Solution_tabulated & normalise()
    {
      get_log().debug("normalise(): start.");

      double partN2 = partial_N2_numerical();
      {
	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "Numerical part of N^2: " << partN2;
	log4cpp::eol(ost);
      }

      double tailN2 = partial_N2_tail();
      {
	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "Tail part of N^2: " << tailN2;
	log4cpp::eol(ost);
      }
      
      double N = std::sqrt(partN2 + tailN2);
      {
	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "Norm is " << N;
	log4cpp::eol(ost);
      }

      // Fill normalised wave-function table
      Solution_tabulated::Solution_table::const_iterator i;
      i = primordial_wf.data().begin();
      while(i != primordial_wf.data().end())
	{
	  double r = (*i).first;
	  double nu = ((*i).second) / N;
	  normalised_wf.push_back(Solution_tabulated::Solution_point(r, nu));
	  i++;
	}

      // Normalised!
      normalised = true;

      get_log().debug("normalise(): finish.");
      return wave_function();
    } // normalise()
  }; // class Normaliser_bound

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_NORMALISER_BOUND_H

// **********************************************
// End of file.
// **********************************************
