/*
 * normaliser_continuum.h
 * Define class Normaliser_continuum.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_NORMALISER_CONTINUUM_H
#define SCHQ_NORMALISER_CONTINUUM_H

// **********************************************
# include <iostream>
# include <cmath>

# include <gsl/gsl_interp.h>
# include <gsl/gsl_spline.h>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_normaliser_base.h"

// **********************************************
namespace schq
{
  class Normaliser_continuum : public Normaliser_base
  {
  private:
    double r1_val;
    double r2_val;

  public:
    /*
     * Constructor.
     */
  Normaliser_continuum(libconfig::Setting & normaliser,
		       libconfig::Setting & system,
		       libconfig::Setting & wave_f,
		       log4cpp::Category & uplevel_log) :
    Normaliser_base(normaliser, system, wave_f, uplevel_log)
    {
      get_log().debug("Normaliser_continuum(): start.");

      libconfig::Setting & asymptotic_points =
	normaliser["asymptotic_points"];
      r1_val = asymptotic_points["r1"];
      r2_val = asymptotic_points["r2"];

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "r_1 = " << r1();
      log4cpp::eol(ost);
      ost << "r_2 = " << r2();
      log4cpp::eol(ost);

      get_log().debug("Normaliser_continuum(): finish.");
    } // Normaliser_continuum()

    // ****************************************************
    /*
     * Destructor.
     */
    virtual ~Normaliser_continuum()
    {
      // Nothing to do!
    } // ~Normaliser_continuum()

    // ****************************************************
    /*
     * double r1() const
     * double r2() const
     * Returns r_1 or r_2 points.
     */
    double r1() const
    {
      return r1_val;
    }
    double r2() const
    {
      return r2_val;
    }

    // ****************************************************
    /*
     * const Solution_tabulated & normalise()
     * Returns the normalised wave-function.
     */
    const Solution_tabulated & normalise()
    {
      get_log().debug("normalise(): start.");

      const Physical_system_base & sys = system();

      double F1 = sys.Coulomb_wave_F(sys.rho( r1() ));
      double G1 = sys.Coulomb_wave_G(sys.rho( r1() ));
      double F2 = sys.Coulomb_wave_F(sys.rho( r2() ));
      double G2 = sys.Coulomb_wave_G(sys.rho( r2() ));
      double u1 = primordial_wf.interpolate(r1());
      double u2 = primordial_wf.interpolate(r2());
      double x = u1/u2;
      double tg = (x * F2 - F1) / (G1 - x * G2);
      double cs = 1.0 / std::sqrt(1.0 + std::pow(tg, 2));
      double N = cs * (F2 + tg * G2) / u2; // Normalisation coefficient

      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "Normalisation coefficient is " << N;
      log4cpp::eol(ost);

      const Solution_tabulated::Solution_table & prim_table =
	primordial_wf.data();
      Solution_tabulated::Solution_table::const_iterator i =
	prim_table.begin();
      while(i != prim_table.end())
	{
	  double r = (*i).first;
	  double nu = N * ((*i).second);
	  normalised_wf.push_back(Solution_tabulated::Solution_point(r, nu));
	  i++;
	}

      // Normalised!
      normalised = true;

      get_log().debug("normalise(): finish.");
      return normalised_wf;
    } // normalise()
  }; // class Normaliser_continuum

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_NORMALISER_CONTINUUM_H

// **********************************************
// End of file.
// **********************************************
