/*
 * physical_system_base.h
 * Define class Physical_system_base, interface for
 * physical systems with various interaction potentials.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_PHYSICAL_SYSTEM_BASE_H
#define SCHQ_PHYSICAL_SYSTEM_BASE_H

// **********************************************
# include <iostream>
# include <string>
# include <stdexcept>
# include <cmath>

# include <gsl/gsl_sf.h>
# include <gsl/gsl_math.h>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_particle.h"

// **********************************************
namespace schq
{
  class Physical_system_base :
  public Bookworm
  {
  public:
    typedef std::pair<Particle, Particle> Particles_pair;

    // **********************************************
    /*
     * Constructor.
     */
  Physical_system_base(const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "system")
      {
	get_log().debug("Physical_system_base(): start.");
	get_log().debug("Physical_system_base(): finish.");
      } // Physical_system_base()

    // **********************************************
    /*
     * Destructor.
     */
    virtual ~Physical_system_base()
      {
	// Nothing to do.
      } // ~Physical_system_base()

    // **********************************************
    // Look at Physical_system_template for comments
    virtual double alpha() const = 0;
    virtual double dVC(const double r) const = 0;
    virtual double dVN(const double r) const = 0;
    virtual double E() const = 0;
    virtual double eta() const = 0;
    virtual double hc() const = 0;
    virtual double J() const = 0;
    virtual double k0_momentum() const = 0;
    virtual double L() const = 0;
    virtual double momentum() const = 0;
    virtual double mu() const = 0;
    virtual const Particles_pair & particles() const = 0;
    virtual double rho(const double r) const = 0;
    virtual double rho_k0(const double r) const = 0;
    virtual double S() const = 0;
    virtual void set_energy(const double newE) = 0;
    virtual double VC(const double r) const = 0;
    virtual double VN(const double r) const = 0;

    // **********************************************
    /*
     * double x_jl(const double l, const double x) const
     * Returns value of x * j_l(x), where
     * j_l(x) is Spherical Bessel Function.
     * std::runtime_error arised if error detected.
     */
    double x_jl(const double l, const double x) const
    {
      double retval;
      gsl_sf_result result;
      int status = gsl_sf_bessel_Jnu_e(l+0.5, x, &result);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): x_jl(): gsl_sf_bessel_Jnu_e() returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}
      retval = result.val;
      retval *= std::sqrt(M_PI * x / 2.0);
      return retval;
    } // x_jl()

    // **********************************************
    /*
     * double x_jl(const double x) const
     * Returns value of x * j_l(x), where
     * j_l(x) is Spherical Bessel Function.
     * l = L().
     */
    double x_jl(const double x) const
    {
      double retval = x_jl(L(), x);
      return retval;
    } // x_jl()

    // **********************************************
    /*
     * double d_x_jl(const double x) const
     * Returns value of (x * j_l(x))', where
     * j_l(x) is Spherical Bessel Function.
     */
    double d_x_jl(const double x) const
    {
      double retval;
      const double l = L();
      retval = -x_jl(l+1, x);
      retval += (l+1) * x_jl(x) / x;
      return retval;
    } // d_x_jl()

    // **********************************************
    /*
     * double Coulomb_wave_F(const double x) const
     * Returns value of regular Coulomb wave function
     * F_l(\eta, x).
     */
    double Coulomb_wave_F(const double x) const
    {
      double retval;
      gsl_sf_result F;
      gsl_sf_result Fp;
      gsl_sf_result G;
      gsl_sf_result Gp;
      double exp_F;
      double exp_G;
      int status =
	gsl_sf_coulomb_wave_FG_e(eta(), x, L(), 0,
				 &F, &Fp, &G, &Gp, &exp_F, &exp_G);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Coulomb_wave_F(): gsl_sf_coulomb_wave_FG_e() "
	      << "returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}
      retval = F.val;
      return retval;
    } // Coulomb_wave_F()

    // **********************************************
    /*
     * double Coulomb_wave_G(const double x) const
     * Returns value of irregular Coulomb wave function
     * G_l(\eta, x).
     */
    double Coulomb_wave_G(const double x) const
    {
      double retval;
      gsl_sf_result F;
      gsl_sf_result Fp;
      gsl_sf_result G;
      gsl_sf_result Gp;
      double exp_F;
      double exp_G;
      int status =
	gsl_sf_coulomb_wave_FG_e(eta(), x, L(), 0,
				 &F, &Fp, &G, &Gp, &exp_F, &exp_G);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Coulomb_wave_G(): gsl_sf_coulomb_wave_FG_e() "
	      << "returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}
      retval = G.val;
      return retval;
    } // Coulomb_wave_G()

    // **********************************************
    /*
     * double Whittaker_dWdx(const double k, const double m,
     *     const double x) const
     * Returns value of derivative of Whittaker function: dW_{k,m}(x)/dx.
     * For formulas see dlmf.nist.gov/13.3.E22 and dlmf.nist.gov/13.14.E3
     */
    double Whittaker_dWdx(const double k, const double m, const double x) const
    {
      double retval;
      gsl_sf_result U_normal;
      gsl_sf_result U_special;

      int status =
	gsl_sf_hyperg_U_e((m - k + 0.5), (1.0 + 2.0 * m), x, &U_normal);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Whittaker_dWdx(): gsl_sf_hyperg_U_e() "
	      << "returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      status =
	gsl_sf_hyperg_U_e((m - k + 1.5), 2.0 * (1.0 + m), x, &U_special);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Whittaker_dWdx(): gsl_sf_hyperg_U_e() "
	      << "returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      double ex = std::exp(-x/2.0);
      double powx = std::pow(x, (0.5 + m));

      double first = (-0.5) * ex;
      first *= powx;
      first *= U_normal.val;

      double second = ex;
      second *= (m + 0.5) * std::pow(x, (m - 0.5));
      second *= U_normal.val;

      double third = ex;
      third *= powx;
      third *= (k - m - 0.5) * U_special.val;

      retval = first + second + third;
      return retval;
    } // Whittaker_dWdx()

    // **********************************************
    /*
     * double Whittaker_W(const double k, const double m, const double x) const
     * Returns value of Whittaker function W_{k,m}(x).
     * WARNING! Vasily Eremenko change formula for W_{k,m}(x):
     * U(..., x) -> U(..., std::abs(x))
     */
    double Whittaker_W(const double k, const double m, const double x) const
    {
      double retval;
      gsl_sf_result result;
      int status =
	gsl_sf_hyperg_U_e((m - k + 0.5), (1.0 + 2.0 * m),
			  std::abs(x), &result);

      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Whittaker_W(): gsl_sf_hyperg_U_e() "
	      << "returned error!";
	  std::string msg = ost.str();
	  get_log().fatal(msg);
	  throw new std::runtime_error(msg);
	}

      retval = std::exp(-x / 2.0);
      retval *= std::pow(x, (m + 0.5));
      retval *= std::abs(result.val);
      return retval;
    } // Whittaker_W()

    // **********************************************
    /*
     * double dWC(const double r) const
     * Returns value of dW_{-\eta, L+1/2}(2 \rho) / d\rho.
     */
    double dWC(const double r) const
    {
      double retval;
      retval = Whittaker_dWdx(-eta(), (L() + 0.5), (2.0 * rho(r)));
      return retval;
    } // dWC()

    // **********************************************
    /*
     * double WC(const double r) const
     * Returns value of W_{-\eta, L+1/2}(2 \rho).
     */
    double WC(const double r) const
    {
      double retval;
      retval = Whittaker_W(-eta(), (L() + 0.5), (2.0 * rho(r)));
      return retval;
    } // WC()

    // **********************************************
    /*
     * double WCN(const double r) const
     * Returns value of W_{\eta, L+1/2}(-2 \rho).
     */
    double WCN(const double r) const
    {
      double retval;
      retval = Whittaker_W(eta(), (L() + 0.5), (-2.0 * rho(r)));
      return retval;
    } // WCN()
  }; // class Physical_system_base

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_PHYSICAL_SYSTEM_BASE_H

// **********************************************
// End of file.
// **********************************************
