/*
 * physical_system_template.h
 * Define class Physical_system_template, class for
 * physical systems with various interaction potentials.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_PHYSICAL_SYSTEM_TEMPLATE_H
#define SCHQ_PHYSICAL_SYSTEM_TEMPLATE_H

// **********************************************
# include <iostream>
# include <string>
# include <stdexcept>
# include <cmath>

# include <gsl/gsl_sf.h>
# include <gsl/gsl_math.h>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_physical_system_base.h"
# include "schq_physical_constants.h"
# include "schq_potentials_base.h"

// **********************************************
namespace schq
{
  template<class Potentials> class Physical_system_template :
  public Physical_system_base
  {
  private:
    Physical_constants * constants;
    Potentials_base * potentials;
    Particles_pair two_particles;
    double energy;
    double orbital_angmom;
    double spin;
    double total_angmom;
    double reduced_mass;

    // **********************************************
    /*
     * void make_particles(const libconfig::Setting & parts)
     * Initialise particles.
     */
    void make_particles(const libconfig::Setting & parts)
      {
	get_log().debug("make_particles(): start.");

	Particle first;
	Particle second;
	first.set_parameters(parts[0]);
	second.set_parameters(parts[1]);
	particles() = Particles_pair(first, second);

	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "particles.first.mass = "
	    << particles().first.mass();
	log4cpp::eol(ost);
	ost << "particles.first.Z = "
	    << particles().first.Z();
	log4cpp::eol(ost);
	ost << "particles.second.mass = "
	    << particles().second.mass();
	log4cpp::eol(ost);
	ost << "particles.second.Z = "
	    << particles().second.Z();
	log4cpp::eol(ost);

	get_log().debug("make_particles(): finish.");
      } // make_particles()

    // **********************************************
    /*
     * void make_angular_momenta(const libconfig::Setting & angmom)
     * Initialise angular momenta.
     */
    void make_angular_momenta(const libconfig::Setting & angmom)
      {
	get_log().debug("make_angular_momenta(): start.");

	spin = angmom["S"];
	orbital_angmom = angmom["L"];
	total_angmom = angmom["J"];

	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "L = " << L();
	log4cpp::eol(ost);
	ost << "S = " << S();
	log4cpp::eol(ost);
	ost << "J = " << J();
	log4cpp::eol(ost);

	get_log().debug("make_angular_momenta(): finish.");
      } // make_angular_momenta()

    // **********************************************
    /*
     * Particles_pair & particles()
     * Returns NON-CONSTANT reference to two_particles.
     */
    Particles_pair & particles()
    {
      return two_particles;
    }

    // **********************************************
  public:
    /*
     * Constructor.
     */
  Physical_system_template(const libconfig::Setting & system,
			   const log4cpp::Category & uplevel_log) :
    Physical_system_base(uplevel_log)
      {
	get_log().debug("Physical_system_template(): start.");

	constants = new Physical_constants(system["constants"], get_log());
	potentials = new Potentials(system["potentials"], get_log());
	make_particles(system["particles"]);
	energy = system["energy"];
	make_angular_momenta(system["angular_momenta"]);
	{
	  // Calculate reduced mass
	  double m1 = particles().first.mass();
	  double m2 = particles().second.mass();
	  reduced_mass = m1 * m2 / (m1 + m2);
	}

	log4cpp::CategoryStream ost(get_log().infoStream());
	ost << "energy = " << E();
	log4cpp::eol(ost);
	ost << "reduced_mass = " << mu();
	log4cpp::eol(ost);

	get_log().debug("Physical_system_template(): finish.");
      } // Physical_system_template()

    // **********************************************
    /*
     * Destructor.
     */
    ~Physical_system_template()
      {
	delete potentials;
	delete constants;
      } // ~Physical_system_template()

    // **********************************************
    /*
     * double alpha() const
     * Returns thin-structure constant
     */
    double alpha() const
    {
      return constants->alpha();
    }

    // **********************************************
    /*
     * double dVC(const double r) const
     * Returns value ofderivative of Coulomb interaction
     * potential at point r.
     */
    double dVC(const double r) const
    {
      return potentials->dVC(r, *this);
    }

    // **********************************************
    /*
     * double dVN(const double r) const
     * Returns value of derivative of nuclear interaction
     * potential at point r.
     */
    double dVN(const double r) const
    {
      return potentials->dVN(r, *this);
    }

    // **********************************************
    /*
     * double E() const
     * Returns value of energy.
     */
    double E() const
    {
      return energy;
    }

    // **********************************************
    /*
     * double eta() const
     * Returns Coulomb factor (LaTeX notation):
     * Z_1 Z_2 \alpha \mu / k,
     * or
     * Z_1 Z_2 \alpha \mu / \varkappa.
     * About k or \varkappa look at momentum().
     */
    double eta() const
    {
      double retval;
      retval = particles().first.Z();
      retval *= particles().second.Z();
      retval *= alpha();
      retval *= mu() / hc(); // convert to fm.
      retval /= momentum();
      return retval;
    } // eta()

    // **********************************************
    /*
     * double hc() const
     * Returns MeV to fm conversion coefficient.
     */
    double hc() const
    {
      return constants->hc();
    }

    // **********************************************
    /*
     * double J() const
     * Returns value of summary angular momenta
     * J = L + S.
     */
    double J() const
    {
      return total_angmom;
    }

    // **********************************************
    /*
     * double k0_momentum() const
     * Returns value of k0.
     */
    double k0_momentum() const
    {
      return potentials->k0_momentum(*this);
    }

    // **********************************************
    /*
     * double L() const
     * Returns value of orbital angular momenta.
     */
    double L() const
    {
      return orbital_angmom;
    }

    // **********************************************
    /*
     * double momentum() const
     * Returns momentum = \sqrt{2\mu E} or
     * \sqrt{2\mu \varepsilon}; where
     * \varepsilon = -E is binding energy.
     * So, it returns k or \varkappa.
     */
    double momentum() const
    {
      double retval = std::sqrt( 2.0 * mu() * std::abs(E()) );
      retval /= hc(); // converting to fm.
      return retval;
    }

    // **********************************************
    /*
     * double mu() const
     * Returns reduced mass.
     */
    double mu() const
    {
      return reduced_mass;
    }

    // **********************************************
    /*
     * const Particles_pair & particles() const
     * Returns CONSTANT reference to two_particles.
     */
    const Particles_pair & particles() const
    {
      return two_particles;
    }

    // **********************************************
    /*
     * double rho(r) const
     * Returns r * momentum().
     */
    virtual double rho(const double r) const
    {
      double retval = r * momentum();
      return retval;
    } // rho()

    // **********************************************
    /*
     * double rho_k0(r) const
     * Returns r * k0_momentum().
     */
    double rho_k0(const double r) const
    {
      double retval = r * k0_momentum();
      return retval;
    } // rho_k0()

    // **********************************************
    /*
     * double S() const
     * Returns value of summary spin.
     */
    double S() const
    {
      return spin;
    }

    // **********************************************
    /*
     * void set_energy(const double newE)
     * Set system energy to newE.
     */
    void set_energy(const double newE)
    {
      energy = newE;
    } // set_energy()

    // **********************************************
    /*
     * double VC(const double r) const
     * Returns value of Coulomb interaction potential at point r.
     */
    double VC(const double r) const
    {
      return potentials->VC(r, *this);
    }

    // **********************************************
    /*
     * double VN(const double r) const
     * Returns value of nuclear interaction potential at point r.
     */
    double VN(const double r) const
    {
      return potentials->VN(r, *this);
    }
  }; // class Physical_system_template

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_PHYSICAL_SYSTEM_TEMPLATE_H

// **********************************************
// End of file.
// **********************************************
