/*
 * solution_tabulated.h
 * Define class Solution_tabulated.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_SOLUTION_TABULATED_H
#define SCHQ_SOLUTION_TABULATED_H

// **********************************************
# include <stdexcept>
# include <cmath>
# include <utility>
# include <list>

# include <libconfig.h++>

# include <gsl/gsl_errno.h>
# include <gsl/gsl_interp.h>

// **********************************************
namespace schq
{
  class Solution_tabulated
  {
  public:
    typedef std::pair<double, double> Solution_point;
    typedef std::list<Solution_point> Solution_table;

    // ****************************************************
  private:
    Solution_table table;
    bool fixed; // for interpolation using GSL
    double * arguments;
    double * values;
    gsl_interp_type * interpolation_type;
    gsl_interp * interpolator_data;
    gsl_interp_accel * interpolation_accelerator;

    // ****************************************************
    /*
     * void fix_me()
     */
    void fix_me()
    {
      // Checks
      if(is_fixed())
	{
	  return; // Already fixed. Nothing to do.
	}
      if(table.empty())
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Trying to fix the empty solution!";
	  std::string msg = ost.str();
	  throw new std::runtime_error(msg);
	}

      // Fixing solution in aim to use GSL's interpolation instruments
      const Solution_table::size_type sz = table.size();
      arguments = new double[sz];
      values = new double[sz];
      Solution_table::const_iterator i = table.begin();
      int j = 0;
      while(i != table.end())
	{
	  arguments[j] = (*i).first;
	  values[j] = (*i).second;
	  j++;
	  i++;
	}
      interpolator_data = gsl_interp_alloc(interpolation_type, sz);
      gsl_interp_init(interpolator_data, arguments, values, sz);
      interpolation_accelerator = gsl_interp_accel_alloc();
      fixed = true;
    } // fix_me()

    // ****************************************************
    /*
     * void unfix_me()
     */
    void unfix_me()
    {
      if(is_fixed())
	{
	  gsl_interp_accel_free(interpolation_accelerator);
	  gsl_interp_free(interpolator_data);
	  delete values;
	  delete arguments;
	}
      fixed = false;
      arguments = NULL;
      values = NULL;
      interpolator_data = NULL;
      interpolation_accelerator = NULL;
    } // unfix_me()

    // ****************************************************
  public:
    /*
     * Constructor(s).
     */
    Solution_tabulated(gsl_interp_type * interp_type =
		       (gsl_interp_type *) gsl_interp_cspline)
    {
      fixed = false; // Is not fixed... Yet. Set flag manually due to probable
      // trash initialisation.
      unfix_me();
      interpolation_type = interp_type;
    } // Solution_tabulated()

    // ****************************************************
    /*
     * Destructor.
     */
    virtual ~Solution_tabulated()
    {
      unfix_me();
    } // ~Solution_tabulated()

    // ****************************************************
    /*
     * void add_to_setting(libconfig::Setting & s) const
     */
    void add_to_setting(libconfig::Setting & s) const
    {
      if( ! s.isGroup())
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): Trying to add object to the Setting of non-Group type!";
	  std::string msg = ost.str();
	  throw new std::runtime_error(msg);
	}

      // Working
      libconfig::Setting & r = s.add("r", libconfig::Setting::TypeArray);
      libconfig::Setting & u = s.add("u", libconfig::Setting::TypeArray);
      Solution_table::const_iterator i = table.begin();
      while(i != table.end())
	{
	  libconfig::Setting & x = r.add(libconfig::Setting::TypeFloat);
	  libconfig::Setting & y = u.add(libconfig::Setting::TypeFloat);
	  x = (*i).first;
	  y = (*i).second;
	  i++;
	}
    } // add_to_setting()

    // ****************************************************
    /*
     * void clear()
     */
    void clear()
    {
      unfix_me();
      table.clear();
    } // clear()

    // ****************************************************
    /*
     * const Solution_table & data() const
     */
    const Solution_table & data() const
      {
	return table;
      } // data()

    // ****************************************************
    /*
     * bool is_fixed() const
     */
    bool is_fixed()
    {
      return fixed;
    }

    // ****************************************************
    /*
     * double interpolate(const double r)
     * Evaluate solution at point r
     */
    double interpolate(const double r)
    {
      double retval;
      if( ! is_fixed())
	{
	  fix_me();
	}
      int status = gsl_interp_eval_e(interpolator_data, arguments, values,
				     r, interpolation_accelerator, &retval);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): interpolate(): gsl_interp_eval_e() "
	      << "returned error status: "
	      << status << "; r = " << r;
	  std::string msg = ost.str();
	  throw new std::runtime_error(msg);
	}
      return retval;
    } // interpolate()

    // ****************************************************
    /*
     * double interpolate_deriv(const double r)
     * Evaluate derivative of the solution at point r
     */
    double interpolate_deriv(const double r)
    {
      double retval;
      if( ! is_fixed())
	{
	  fix_me();
	}
      int status = gsl_interp_eval_deriv_e(interpolator_data, arguments,
					   values, r,
					   interpolation_accelerator, &retval);
      if(status != GSL_SUCCESS)
	{
	  std::ostringstream ost;
	  ost << __FILE__ << "(" << __LINE__
	      << "): interpolate_deriv(): gsl_interp_eval_deriv_e() "
	      << "returned error status: "
	      << status << "; r = " << r;
	  std::string msg = ost.str();
	  throw new std::runtime_error(msg);
	}
      return retval;
    } // interpolate_deriv()

    // ****************************************************
    /*
     * void push_back(const Solution_point & x)
     */
    void push_back(const Solution_point & x)
    {
      if(is_fixed())
	{
	  unfix_me();
	}
      table.push_back(x);
    } // push_back()

    // ****************************************************
    /*
     * std::ostream & put(std::ostream & s) const
     * Write object to given stream.
     */
    std::ostream & put(std::ostream & s) const
      {
	Solution_table::const_iterator i = table.begin();
	while(i != table.end())
	  {
	    s << (*i).first << "\t" << (*i).second << std::endl;
	    i++;
	  }
	return s;
      } // put()

    // ****************************************************
    /*
     * void read_from_setting(libconfig::Setting & wf)
     */
    void read_from_setting(libconfig::Setting & wf)
    {
      libconfig::Setting & r = wf["r"];
      libconfig::Setting & u = wf["u"];
      int i;
      for(i = 0; true; i++)
	{
	  try
	    {
	      double x = r[i];
	      double y = u[i];
	      push_back(Solution_point(x, y));
	    }
	  catch(libconfig::SettingNotFoundException)
	    {
	      // End of arrays is reached
	      break;
	    }
	}
    } // read_from_setting()
  }; // class Solution_tabulated
} // namespace schq

// **********************************************
/*
 * std::ostream & operator<< (std::ostream & s,
 *                            const schq::Solution_tabulated & l)
 */
std::ostream & operator<< (std::ostream & s,
			   const schq::Solution_tabulated & l)
{
  return l.put(s);
} // operator<< ()

// **********************************************
#endif // SCHQ_SOLUTION_TABULATED_H

// **********************************************
// End of file.
// **********************************************
