/*
 * solver_base.h
 * Define class Solver_base.
 *
 * Copyright (c) 2010 Vasily Eremenko <vasily.eremenko@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// **********************************************
#ifndef SCHQ_SOLVER_BASE_H
#define SCHQ_SOLVER_BASE_H

// **********************************************

# include <string>
# include <stdexcept>
# include <typeinfo>
# include <cmath>
# include <list>

# include <gsl/gsl_errno.h>
# include <gsl/gsl_matrix.h>
# include <gsl/gsl_odeiv.h>
# include <gsl/gsl_interp.h>
# include <gsl/gsl_spline.h>

# include <libconfig.h++>

# include <log4cpp/Category.hh>

# include "schq_bookworm.h"
# include "schq_physical_system_base.h"
# include "schq_solution_tabulated.h"

// **********************************************
namespace schq
{
  // Declare functions
  int calculate_jacobian_schroedinger_eq(
					 const double,
					 const double[],
					 double*,
					 double[],
					 void*);

  int calculate_right_part_schroedinger_eq(
					   const double,
					   const double[],
					   double[],
					   void*);

  // **********************************************
  class Solver_base : public Bookworm
  {
    friend int calculate_jacobian_schroedinger_eq(
						  const double,
						  const double[],
						  double*,
						  double[],
						  void*);

    friend int calculate_right_part_schroedinger_eq(
						    const double,
						    const double[],
						    double[],
						    void*);

    // ****************************************************
  private:
    Physical_system_base * ph_sys;
    std::pair<double, double> interval;
    double istep_size;
    double step_error_abs_val;
    double step_error_rel_val;
    Solution_tabulated solution_val;
    bool solved;

    // ****************************************************
    /*
     * double initial_step_size() const
     */
    double initial_step_size() const
    {
      return istep_size;
    }

    // ****************************************************
    /*
     * double step_error_abs() const
     */
    double step_error_abs() const
    {
      return step_error_abs_val;
    }

    // ****************************************************
    /*
     * double step_error_rel() const
     */
    double step_error_rel() const
    {
      return step_error_rel_val;
    }

    // ****************************************************
  public:
    /*
     * Constructor.
     */
  Solver_base(Physical_system_base * sys,
	      const libconfig::Setting & numeric,
	      const log4cpp::Category & uplevel_log) :
    Bookworm(uplevel_log, "solver")
    {
      get_log().debug("Solver_base(): start.");

      ph_sys = sys;
      solved = false;

      libconfig::Setting & intrvl = numeric["interval"];
      interval.first = intrvl["from"];
      interval.second = intrvl["to"];
      istep_size = numeric["initial_step_size"];

      libconfig::Setting & stp = numeric["step_error"];
      step_error_abs_val = stp["absolute"];
      step_error_rel_val = stp["relative"];

      // Final print
      log4cpp::CategoryStream ost(get_log().infoStream());
      ost << "r_min = " << r_min();
      log4cpp::eol(ost);
      ost << "r_max = " << r_max();
      log4cpp::eol(ost);
      ost << "initial step size = " << initial_step_size();
      log4cpp::eol(ost);
      ost << "Absolute step error limit = "
	  << step_error_abs();
      log4cpp::eol(ost);
      ost << "Relative step error limit = "
	  << step_error_rel();
      log4cpp::eol(ost);
      ost << "VN(r_min) = " << system().VN(r_min());
      log4cpp::eol(ost);
      ost << "VC(r_min) = " << system().VC(r_min());
      log4cpp::eol(ost);

      get_log().debug("Solver_base(): finish.");
    } // Solver_base()

    // ****************************************************
    /*
     * Destructor.
     */
    virtual ~Solver_base()
    {
      // Nothing to do.
    } // ~Solver_base()

    // ****************************************************
    /*
     * void clear()
     */
    void clear()
    {
      solution_val.clear();
      solved = false;
    }

    // ****************************************************
    /*
     * const bool is_solved() const
     */
    const bool is_solved() const
    {
      return solved;
    }

    // ****************************************************
    /*
     * const Physical_system_base & system() const
     */
    const Physical_system_base & system() const
    {
      return *ph_sys;
    }

    // ****************************************************
    /*
     * Solution_tabulated & solution()
     * Returns the solution of Schroedinger equation.
     */
    Solution_tabulated & solution()
    {
      return solution_val;
    }

    // ****************************************************
    /*
     * double r_min() const
     * Returns left border of interval.
     */
    double r_min() const
    {
      return interval.first;
    } // r_min()

    // ****************************************************
    /*
     * double r_max() const
     * Returns right border of interval.
     */
    double r_max() const
    {
      return interval.second;
    } // r_max()

    // ****************************************************
    /*
     * const gsl_odeiv_step_type * algorythm() const
     * Fixes the algorythm to solve Schroedinger equation.
     */
    virtual const gsl_odeiv_step_type * algorythm() const = 0;

    // ****************************************************
    /*
     * std::pair<double, double> initial_values() const
     * Initial values of u(r) and u'(r). Reload this function in derivative
     * classes to set other values.
     */
    virtual std::pair<double, double> initial_values() const
    {
      const double k0 = system().k0_momentum();
      const double rho = system().rho_k0(r_min());
      double u = system().x_jl(rho);
      double du;
      du = k0 * system().d_x_jl(rho);
      std::pair<double, double> retval(u, du);
      return retval;
    }

    // ****************************************************
    /*
     * Solution_tabulated & solve()
     * Solve Schrodinger equation.
     */
    virtual Solution_tabulated & solve()
    {
      get_log().debug("solve(): start.");

      // Preparations
      const gsl_odeiv_step_type * alg = algorythm();
      gsl_odeiv_step * stp = gsl_odeiv_step_alloc(alg, 2);
      gsl_odeiv_control * ctrl =
	gsl_odeiv_control_y_new(step_error_abs(),
				step_error_rel());
      gsl_odeiv_evolve * ev = gsl_odeiv_evolve_alloc(2);

      // System
      gsl_odeiv_system sys =
	{
	  calculate_right_part_schroedinger_eq,
	  calculate_jacobian_schroedinger_eq,
	  2,
	  this
	};

      double h = initial_step_size();
      std::pair<double, double> iv = initial_values();
      double y[2] = {iv.first, iv.second}; // Initial values
      double r;
      double left = r_min();
      double right = r_max();

      // Solving
      r = left;
      while(r < right)
	{
	  solution_val.push_back(Solution_tabulated::Solution_point(r, y[0]));
	  int status = gsl_odeiv_evolve_apply(ev, ctrl, stp,
					      &sys, &r, right,
					      &h, y);
	  if(status != GSL_SUCCESS)
	    {
	      std::ostringstream ost;
	      ost << __FILE__ << "(" << __LINE__
		  << "): solve(): error detected at r = "
		  << r;
	      std::string msg = ost.str();
	      get_log().fatal(msg);
	      throw new std::runtime_error(msg);
	    }
	} // while()
      solution_val.push_back(Solution_tabulated::Solution_point(r, y[0]));

      // Cleanup
      gsl_odeiv_evolve_free(ev);
      gsl_odeiv_control_free(ctrl);
      gsl_odeiv_step_free(stp);

      // Solved!
      solved = true;

      get_log().debug("solve(): finish.");
      return solution();
    } // solve()
  }; // class Solver_base

  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // **********************************************
  // Define functions
  int calculate_right_part_schroedinger_eq(
					   const double r,
					   const double y[],
					   double f[],
					   void * params)
  {
    const Solver_base & caller = *(const Solver_base*)params;
    const double hc = caller.system().hc();
    const double mu = caller.system().mu();
    const double l = caller.system().L();
    // Working
    f[0] = y[1];
    double f1;
    f1 = 2.0 * mu * y[0] / std::pow(hc, 2);
    f1 *= caller.system().VN(r) + caller.system().VC(r) +
      std::pow(hc, 2) * l * (l + 1) / (2.0 * mu * std::pow(r, 2)) -
      caller.system().E();
    f[1] = f1;
    return GSL_SUCCESS;
  } // calculate_right_part_schroedinger_eq()

  // **********************************************
  int calculate_jacobian_schroedinger_eq(
					 const double r,
					 const double y[],
					 double *dfdy,
					 double dfdt[],
					 void * params)
  {
    const Solver_base & caller = *(const Solver_base*)params;
    const double hc = caller.system().hc();
    const double mu = caller.system().mu();
    const double l = caller.system().L();
    // dfdy
    gsl_matrix_view dfdy_mat = gsl_matrix_view_array(dfdy, 2, 2);
    gsl_matrix * m = &dfdy_mat.matrix;
    gsl_matrix_set(m, 0, 0, 0.0);
    gsl_matrix_set(m, 0, 1, 1.0);
    gsl_matrix_set(m, 1, 0, 1.0);
    gsl_matrix_set(m, 1, 1, 0.0);
    // dfdt
    dfdt[0] = 0.0;
    double dfdt1 = y[0] * 2.0 * mu / std::pow(hc, 2);
    dfdt1 *= caller.system().dVN(r) + caller.system().dVC(r) -
      std::pow(hc, 2) * l * (l + 1) / (mu * std::pow(r, 3));
    dfdt[1] = dfdt1;
    return GSL_SUCCESS;
  } // calculate_jacobian_schroedinger_eq()

  // **********************************************
} // namespace schq

// **********************************************
#endif // SCHQ_SOLVER_BASE_H

// **********************************************
// End of file.
// **********************************************
